// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 rgsimageforbadpixfind

   Authors:  A. Ibarra - XMM-Newton SOC
         
   Description: Creates a RGS image for rgsbadpixfind
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/

#include "rgsimageforbadpixfind.h"

#include <Tracer.h>
#include <errstr.h>
#include <vector>
#include <string>
#include <cmath>
#include <unistd.h>
#include <Dal.h>
#include <param.h>
#include "dal_utilities.h"
#include "DataFilter.h"
#include "Extractor.h"


using namespace std;

class rgsImage : public Product {

public:

	rgsImage(const std::string & name, Table *intable,
	bool destructive, const std::string &flagColumn, int flagBit)
    : Product(name, intable, destructive, flagColumn, flagBit) {};

	void createImage(Table *tab)
	{

		//filter file: One rgs image for each node

		int node = intParameter("node");
		std::stringstream expr;
		expr << "CCDNODE == "<< node;
		DataFilter obsFilter(expr.str());

		errstr << "Filter expression:  " << expr.str() <<  Sas::message(Msg::App, Msg::Sparse);


		obsFilter.doFilter(*tab);

		std::vector<real64> binSize(2);

		std::vector<long unsigned> size;
		size.push_back(128);
		size.push_back(171);

		Column *xCol, *yCol = 0;
		ColumnData *xColDat, *yColDat = 0;


		std::string xcolumn = "RAWX";
		xCol = tab->column(xcolumn);

		xColDat = xCol->data();
		int16 *xData = 0;
		xData = xColDat->int16Data();
		std::string ycolumn = "RAWY";
		yCol = tab->column(ycolumn);
		yColDat = yCol->data();
		int16 *yData = 0;
		yData = yColDat->int16Data();


		//Xlimits and YLimits

		std::vector<real64> xLimits(2), yLimits(2);
		real64 minRange,maxRange;

		getRange(xCol, minRange, maxRange);

		cout << "minRange " << minRange << endl;
		xLimits[0] = minRange;
		xLimits[1] = maxRange;
		xLimits[0] -= 0.5;
		xLimits[1] += 0.5;


		getRange(yCol, minRange, maxRange);
		yLimits[0] = minRange;
		yLimits[1] = maxRange;

		yLimits[0] -= 0.5;
		yLimits[1] += 0.5;


		binSize[0] = ceil( (yLimits[1]-yLimits[0])/float(size[0]) );
		binSize[1] = ceil( (xLimits[1]-xLimits[0])/float(size[1]) );


		//Create image
		//DataSet *dataSet;
		//_dataSet = dataSetServer->open("out.ds", DataSet::Create);



		Array * outImage = _dataSet->addArray("PRIMARY", Dal::Int32, size);

		// Accumulate the image.
		ArrayData * cImageData = outImage->data();
		int32 *imageData = cImageData->int32Data();
		for(unsigned im=0; im<size[0]*size[1]; im++) imageData[im]=0;

		unsigned long mRows = tab->rows();

		for(unsigned long irow=0; irow < mRows; irow++)
		{
			if( xData[irow]<xLimits[0] || xData[irow]>xLimits[1] )continue;
			if( yData[irow]<yLimits[0] || yData[irow]>yLimits[1] )continue;

			int ix = (int)floor( (xData[irow]-xLimits[0])/binSize[1] );
			int iy = (int)floor( (yData[irow]-yLimits[0])/binSize[0] );

			imageData[iy*size[1]+ix] += 1;
		}

		//Copy keywords

		cout << "KEYWORDS " << endl;
		_copyGlobalAttribs(outImage);



		* outImage->addAttribute( "INSTRUME",
						"instrument")
			    		  = tab->attribute("INSTRUMENT")->asString();


		* outImage->addAttribute( "CCDNODE",
						"node")
			    		  = node;

		* outImage->addAttribute( "ABUTTED",
						"abuts on paried node")
			    		  = true;

		* outImage->addAttribute( "HDUCLASS",
				"Format conforms to OGIP/GSFC conventions")
	    		  = "OGIP";
		* outImage->addAttribute( "HDUCLAS1",
				"File contains an Image")
	    		  = "IMAGE";

		* outImage->addAttribute( "HDUCLAS2",
				"Gross Image")
	    		  = "TOTAL";

		* outImage->addAttribute( "HDUVERS1",
				"Version of format")
	    		  = "1.1.0";

		cout << "KEYWORDS2 " << endl;
		//Check for and set WCS keywords



		//close
		delete xColDat;
		delete yColDat;
		delete cImageData;
		//dataSetServer->close(dataSet);


	}


private:


};

void rgsimageforbadpixfind ()
{
	PROC
	//REad evlist
	string evtfile = stringParameter("table");
	stringstream tmpname;
	tmpname << tmpnam(0) << ".ds";
	dataSetServer->copy(evtfile, tmpname.str());
	DataSet *set = dataSetServer->open(tmpname.str(), Dal::Modify);
	Table *tab = set->table("PIXELS");

	int node = intParameter("node");
	string imgfile = stringParameter("imageset");

	rgsImage rgs(imgfile,tab,true,"EVFLAG",-1);
	rgs.createImage(tab);

	//dataSetServer->close(set);
	unlink(tmpname.str().c_str());
	ENDPROC
}
